import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.LookupDiscovery;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;
import java.util.Vector;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;

public class Client implements Runnable{
    protected ServiceTemplate template;
    protected LookupDiscovery disco;
  
    //An inner class to implement DiscoveryListener
    class Listener implements DiscoveryListener{
        public void discovered(DiscoveryEvent ev){
            ServiceRegistrar[] newregs = ev.getRegistrars();
            for (int i=0 ; i<newregs.length ; i++){
                lookForService(newregs[i]);
			}
		}
        public void discarded(DiscoveryEvent ev){}
    }
  
    public Client() throws IOException{
        Class[] types= {RemoteObjectFactory.class};
		
        template = new ServiceTemplate(null, types, null);
		
        //Set a security manager
        if (System.getSecurityManager() == null){
            System.setSecurityManager(new RMISecurityManager());
		}
		
        //Only serch the public group
        disco = new LookupDiscovery(new String[] {""});
		
        //Install a listener
        disco.addDiscoveryListener(new Listener());
    }//end of Client()
	
    //Once we've found a new lookup service, serch for proxies
    //that implement RemoteObjectFactoryInterface
    protected Object lookForService(ServiceRegistrar lusvc){
        Object o = null;
		
        try {
            o = lusvc.lookup(template);
        } catch (RemoteException ex) {
            System.out.println("Error doing lookup: " + ex.getMessage());
            return null;
        }
		
        if (o == null) {
            System.out.println("No matching service.");
            return null;
        }
		
        System.out.println("Got a matching service.....");
		
        //*********** call createObject here ************
        try {
            RemoteOutputStream out = new RemoteOutputStream(System.out);
            RemoteOutputStream err = new RemoteOutputStream(System.err);
            MyClass mc = (MyClass)((RemoteObjectFactory)o).createObject(out, 
                                                                        err);
            mc.speak(arg);
        }catch (RemoteException e){
			System.out.println("RemoteObjectFactory error: "+e);
			System.exit(0);
		}
        return o;
    }

    //This thread does nothing--it simply keeps the VM from 
    //exiting while we do discovery.
    public void run(){
        while (true) {
          try {
              Thread.sleep(1000000);
          }catch (InterruptedException ex){}
        }
    }

    static String arg;
  
    //Create a Client and start its thread
    public static void main(String args[]){
      try {
          arg = args[0];

          Client hwc = new Client();
          new Thread(hwc).start();
      }catch (IOException ex) {
          System.out.println("Couldn't create client: " + ex.getMessage());
      }
    }
}//end of class Client
